;
;
;  ayfx mini ver.0.1		copyright (c) 2014 introspec
;
;  shiru wrote what he called 'minimal ayfx player' for the playback of sounds
;  on ay/ym-compatible psgs. his player is actually quite sophisticated in many
;  situations, esp. when one attempts to keep the music playing alongside the
;  sound effects. this is an attempt at an alternative, less intrusive playback
;  strategy. it is sophisticated in a different way, because it attempts to
;  co-exist with a simultaneously-playing psg tune.
;
;  ver.0.1 (re-write of the ayfx player v0.15 by shiru)
;

			DEFINE	BUFFERED_PLAYER	; the only working option at this stage (do not comment out)

FX_CHANNEL:		EQU	2		; channel to use for all outputs (0=A, 1=B, 2=C)

			MODULE	MiniAYFX

AYToneReg:		EQU	16+0+2*FX_CHANNEL
AYNoiseReg:		EQU	16+6
AYMixerReg:		EQU	16+7
AYVolReg:		EQU	16+8+FX_CHANNEL


;
;  initiate the playback of a new effect
;  A = the effect number (0..255)
;

@AFXPLAY:		; determine the address with effect data
CurFXTable:		ld	hl, 0
			ld	c, a
			ld	b, 0
			add	hl, bc
			add	hl, bc
			ld	c, (hl)
			inc	hl
			ld	b, (hl)
			add	hl, bc

			; update the playback code, if necessary
			ld	a, (CurFXAddr+2)
			or	a
			jr	nz, AlreadyPlaying

			; switch fx on in the playback routine
			ld	a, #E5		; second byte of SET 4, L
			ld	(SwitchFX1+1), a
			ld	(SwitchFX2+1), a
			ld	(SwitchFX3+1), a

AlreadyPlaying:		ld	(CurFXAddr+1), hl
			ret

;
;  playback of one new frame
;

@AFXFRAME:
@CurFXAddr:		ld	hl, 0
			ld	a, h
			or	a
			ret	z

			; if the musical player updated one or several registers, we need
			; to be able to restore their correct values after fx finished playing
                        IFDEF	BUFFERED_PLAYER

			ld	de, PLAYER+AYVolReg	; register values for the next frame

			ld	c, (hl)	; read commands for the next frame
			inc	hl

			; lowest 4 bits of A are the current effect volume, which is updated every frame
			ld	a, c
			and	#0F
			ld	(de), a

			; bit 5 of A tells us if the current channel's frequency needs to be modified
			bit	5, c
			jr	z, KeepSameFreq

			ld	a, c
			ld	e, AYToneReg
			ldi
			ldi
			ld	c, a

KeepSameFreq:		; bit 6 of A tells us if the current noise period needs to be modified
			bit	6, c
			jr	z, KeepSameNoise

			ld	a, (hl)
			sub	#20
			jr	nc, TheEndOfFX

			inc	hl
			ld	e, AYNoiseReg	; noise register
			ld	(de), a

KeepSameNoise:		; finally, the mixer must be updated during each frame
			; bits 7 and 4 of E store the mixer state
			ld	a, c
			DUP	4-FX_CHANNEL
			rrca
			EDUP
			ld	c, a

			ld	e, AYMixerReg-16
			ld	a, (de)
			xor	c
			and	low (%1111011011>>(2-FX_CHANNEL))	; the corresponding mask
			xor	c
			ld	e, AYMixerReg
			ld	(de), a

			ld	(CurFXAddr+1), hl
			ret

;
;  player initialisation (setting up the effects table address and switching off the sound)
;

@AFXINIT:		inc	hl
			ld	(CurFXTable+1), hl

TheEndOfFX:		; switch fx off in the playback routine
			ld	a, #A5		; second byte of RES 4, L
			ld	(SwitchFX1+1), a
			ld	(SwitchFX2+1), a
			ld	(SwitchFX3+1), a

			xor	a
			ld	(CurFXAddr+2), a
			ret

			ELSE

			; i do not actually know how to achieve the same functionality, but without a buffered player!

			ENDIF

			ENDMODULE


